# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .customer_status import CustomerStatus


class CustomerBase(UniversalBaseModel):
    name: str
    description: typing.Optional[str] = None
    metadata: typing.Dict[str, str]
    status: CustomerStatus = pydantic.Field()
    """
    Whether or not the customer is in active use.
    
    Only active customers will be available for agent interactions.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
