# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .bot_logic_action_executed_detail import BotLogicActionExecutedDetail
from .bot_logic_action_reviewed_detail import BotLogicActionReviewedDetail


class BotLogicActionItem(UniversalBaseModel):
    actions_reviewed: typing_extensions.Annotated[
        typing.List[BotLogicActionReviewedDetail], FieldMetadata(alias="actionsReviewed")
    ]
    actions_executed: typing_extensions.Annotated[
        typing.List[BotLogicActionExecutedDetail], FieldMetadata(alias="actionsExecuted")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
