# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .customer_status import CustomerStatus


class CustomerFilter(UniversalBaseModel):
    search: typing.Optional[str] = pydantic.Field(default=None)
    """
    The search query to apply to the customers' name and description.
    """

    statuses: typing.Optional[typing.List[CustomerStatus]] = pydantic.Field(default=None)
    """
    The statuses to filter the customers by.
    """

    assignees: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter customers by assignee agent user IDs. Returns customers that have any of the specified assignees.
    """

    members: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter customers by member agent user IDs. Returns customers that have any of the specified members.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
